package com.ejie.ab60.control.convocatorias;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.ab60.model.Idiomas;
import com.ejie.ab60.service.convocatorias.IdiomasService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * IdiomasController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/idiomas")

public class IdiomasController {

	private static final Logger logger = LoggerFactory.getLogger(IdiomasController.class);

	@Autowired
	private IdiomasService idiomasService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idIdioma
	 *            Long
	 * @return Idiomas Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idIdioma}", method = RequestMethod.GET)
	public @ResponseBody Idiomas get(@PathVariable Long idIdioma) {
		Idiomas idiomas = new Idiomas();
		idiomas.setIdIdioma(idIdioma);
		idiomas = this.idiomasService.find(idiomas);
		IdiomasController.logger.info("[GET - findBy_PK] : Obtener Idiomas por PK");
		return idiomas;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterIdiomas
	 *            Idiomas Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Idiomas> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Idiomas> getAll(@ModelAttribute Idiomas filterIdiomas) {
		IdiomasController.logger.info("[GET - find_ALL] : Obtener Idiomas por filtro");
		return this.idiomasService.findAll(filterIdiomas, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param idiomas
	 *            Idiomas Bean que contiene la informacion a modificar.
	 * @return Idiomas Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Idiomas edit(@RequestBody Idiomas idiomas) {
		Idiomas idiomasAux = this.idiomasService.update(idiomas);
		IdiomasController.logger.info("[PUT] : Idiomas actualizado correctamente");
		return idiomasAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param idiomas
	 *            Idiomas Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Idiomas Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Idiomas add(@RequestBody Idiomas idiomas) {
		Idiomas idiomasAux = this.idiomasService.add(idiomas);
		IdiomasController.logger.info("[POST] : Idiomas insertado correctamente");
		return idiomasAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idIdioma
	 *            Long Identificador del objeto que se desea eliminar.
	 * @return Idiomas Bean eliminado.
	 */
	@RequestMapping(value = "/{idIdioma}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Idiomas remove(@PathVariable Long idIdioma) {
		Idiomas idiomas = new Idiomas();
		idiomas.setIdIdioma(idIdioma);
		this.idiomasService.remove(idiomas);
		IdiomasController.logger.info("[DELETE] : Idiomas borrado correctamente");
		return idiomas;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		IdiomasController.logger.info("[GET - View] : idiomas");
		return "idiomas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterIdiomas
	 *            Idiomas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Idiomas> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Idiomas> filter(@RequestJsonBody(param = "filter") Idiomas filterIdiomas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		IdiomasController.logger.info("[POST - filter] : Obtener Idiomass");
		return this.idiomasService.filter(filterIdiomas, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterIdiomas
	 *            Idiomas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchIdiomas
	 *            Idiomas Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Idiomas> Dto que contiene el resultado de la busqueda
	 *         realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Idiomas>> search(@RequestJsonBody(param = "filter") Idiomas filterIdiomas,
			@RequestJsonBody(param = "search") Idiomas searchIdiomas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		IdiomasController.logger.info("[POST - search] : Buscar Idiomass");
		return this.idiomasService.search(filterIdiomas, searchIdiomas, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterIdiomas
	 *            Idiomas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Idiomas filterIdiomas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		IdiomasController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Idiomass");
		this.idiomasService.removeMultiple(filterIdiomas, jqGridRequestDto, false);
		IdiomasController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterIdiomas
	 *            Idiomas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Idiomas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Idiomas>> jerarquia(
			@RequestJsonBody(param = "filter") Idiomas filterIdiomas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		IdiomasController.logger.info("[POST - jerarquia] : Obtener Idiomass jerarquia");
		return this.idiomasService.jerarquia(filterIdiomas, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterIdiomas
	 *            Idiomas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Idiomas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Idiomas>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Idiomas filterIdiomas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		IdiomasController.logger.info("[POST - jerarquia] : Obtener Idiomass jerarquia - Hijos");
		return this.idiomasService.jerarquiaChildren(filterIdiomas, jqGridRequestDto);
	}

	/**
	 * Metodo que nos trae la lista para cargar el combo de idiomas
	 * 
	 * 
	 */
	@RequestMapping(value = "/verValorCombo", method = RequestMethod.GET)
	public @ResponseBody List<Idiomas> init() {
		IdiomasController.logger.info("[GET - getValorCombo] : Obtener valores del combo ");
		return this.idiomasService.findIdioma();

	}

}
